/*
 * gexiv2-metadata.h
 *
 * Author(s)
 * 	Jim Nelson <jim@yorba.org>
 *
 * This is free software. See COPYING for details.
 */

#ifndef __GEXIV2_H__
#define __GEXIV2_H__

#include <gexiv2/gexiv2-metadata.h>
#include <gexiv2/gexiv2-managed-stream.h>
#include <gexiv2/gexiv2-preview-properties.h>
#include <gexiv2/gexiv2-preview-image.h>

#endif /* __GEXIV2_H__ */
